@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use 5.010;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use PPI;
 use PPI::Document;
@@ -28,6 +28,7 @@ sub __const_check {
     );
     return unless $found;
 
+    my $obj_arguments = {};
     my $original_code = $code;
     $code = "";
     for my $op_range (@$found) {
@@ -43,7 +44,46 @@ sub __const_check {
             $pnode = $pnode->parent;
         }
 
-        $code .= ($start ? $start->content : "") . "+" . "PerlX::Range->new(last => @{[ $end->content ]})";
+        my $end_content = $end->content;
+
+        if ($end_content eq '*') {
+            $end_content = '"*"';
+        }
+        elsif ($end_content eq '*:') {
+            my $selector = $end->snext_sibling;
+            say "XXX: $selector";
+            if ($selector->content eq 'by') {
+                my $selector_arg = $selector->snext_sibling;
+                say "XXX: $selector_arg";
+            }
+            else {
+                die("Unknown Range syntax: $selector");
+            }
+            $end_content = '"*"';
+        }
+        else {
+            my $colon = $end->snext_sibling;
+            if ($colon->content eq ':') {
+                my $selector = $colon->snext_sibling;
+                if ($selector && $selector->content eq 'by') {
+                    my $selector_arg = $selector->snext_sibling;
+                    if ($selector_arg && "$selector_arg" =~ /\((\d+)\)/) {
+                        $obj_arguments->{by} = $1;
+                    }
+                }
+                else {
+                    die("Unknown Range syntax: $selector");
+                }
+            }
+        }
+
+        $obj_arguments->{last} = $end_content;
+
+        my $argument_string = "";
+        for (keys %$obj_arguments) {
+            $argument_string .= "$_ => " . $obj_arguments->{$_} . ", ";
+        }
+        $code .= ($start ? $start->content : "") . "+" . "PerlX::Range->new($argument_string)";
     }
 
     substr($linestr, $offset, length($original_code) - 2 ) = $code;
@@ -59,6 +99,11 @@ sub import {
 }
 
 use overload
+    '@{}' => sub {
+        my $self = shift;
+        my @a = $self->to_a;
+        return \@a;
+    },
     '+' => sub {
         my $self = shift;
         $self->{first} = $_[0];
@@ -89,20 +134,23 @@ sub last {
 *min = *from = \&first;
 *max = *to   = \&last;
 
+sub to_a {
+    my $self = shift;
+    my @r = ();
+    $self->each(sub { push @r, $_ });
+    return @r;
+}
+
 sub each {
     my $cb = pop;
     my $self = shift;
 
-    my $current = $self->{current} ||= $self->{first};
-    if ($current > $self->{last}) {
-        delete $self->{current};
-        return;
-    }
-    while($current <= $self->{last}) {
+    my $current = $self->min;
+    while('*' eq $self->max || $current <= $self->max) {
         local $_ = $current;
         my $ret = $cb->($self, $_);
         last if (defined($ret) && !$ret);
-        $current++;
+        $current += $self->{by} ? $self->{by} : 1;
     }
 }
 
@@ -135,6 +183,20 @@ PerlX::Range is an attemp to implement make range operator lazy. When you say:
 
 This `$a` variable is then now a C<PerlX::Range> object.
 
+At this point the begin of range can only be a constant, and better
+only be a number literal.
+
+The end of the range can be a number, or a asterisk C<*>, which means
+"whatever", or Inf. This syntax is stolen from Perl6.
+
+After the end of range, it optionally take a C<:by(N)> modifier, where
+N can be a number literal. This syntax is also stolen from Perl6.
+
+Therefore, this is how you represent all odd numbers:
+
+    my $odd = 1..*:by(2);
+
+
 =head1 METHODS
 
 =over 4
@@ -152,6 +214,10 @@ Retrieve the maximum value of the range.
 Iterate over the range one by one, the C<$cb> should be a code
 ref. Inside the body of that, C<$_> refers to the current value.
 
+If you want to stop before it reach the end of the range, or you have
+to because the range is infinite, you need to say C<return 0>. A
+defined false value from C<$cb> will make the iteration stop.
+
 =back
 
 =head1 AUTHOR
